#![cfg_attr(not(test), no_std)]
#![allow(deprecated)]

use evse_radio_messages::{Direction, DownlinkMessage, UplinkMessage, RadioDatagram, SeqExtra};
use postcard::to_vec;
use heapless::consts::*;
use core::convert::TryInto;

pub use evse_radio_messages::Channel;

pub struct Message {
    pub id: u8,
    pub data: u32,
    pub channel: Channel,
}

#[derive(Debug)]
pub struct Error(&'static str);

impl Message {
    pub fn serialize(&self) -> Result<heapless::Vec<u8, U90>, Error> {
        let packet = UplinkMessage::Ltu {
            channel: self.channel,
            data: self.data,
        };
        let payload: heapless::Vec<u8, U90> = if let Ok(payload) = to_vec(&packet) {
            payload
        } else {
            return Err(Error("Could not serialize packet payload"));
        };
        let datagram = RadioDatagram {
            did: (self.id as u32).try_into().unwrap(),
            seq: SeqExtra::new(0, Direction::Uplink),
            payload: &payload,
            mac: 0x4321_abcd,
        };
        let binary = if let Ok(binary) = to_vec(&datagram) {
            binary
        } else {
            return Err(Error("Could not serialize datagram"));
        };
        Ok(binary)
    }

    pub fn deserialize(packet: &[u8]) -> Option<Self> {
        if !mac(packet) {
            return None;
        }

        let datagram: RadioDatagram = if let Ok(datagram) = postcard::from_bytes(packet) {
            datagram
        } else {
            return None;
        };
        
        if let Direction::Uplink = datagram.seq.direction() {
            return None;
        }

        let id: u32 = match datagram.did.try_into() {
            Ok(id) => id,
            Err(_) => return None,
        };
        let id: u8 = match id.try_into() {
            Ok(id) => id,
            Err(_) => return None,
        };
        match postcard::from_bytes(&datagram.payload) {
            Ok(DownlinkMessage::Ltu {channel, data}) => Some(Message { id, channel, data }),
            _ => None,
        }
    }
}

fn mac(payload: &[u8]) -> bool {
    if payload.len() < 4 {
        return false;
    }
    let mac: u32 = if let Some(mac) = payload.get(payload.len() - 4..) {
        let mut array: [u8; 4] = [0; 4];
        array.copy_from_slice(mac);
        u32::from_le_bytes(array)
    } else {
        return false;
    };
    mac == 0x4321_abcd
}